<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use DateTime;

class Tco01101DetalhesServico {
    protected $environment;
    protected $entityManager;
    protected $tco01101DetalhesServicoService;
    
    public function __construct($environment, $entityManager, $tco01101DetalhesServicoService) {
        $this->environment                       = $environment;
        $this->entityManager                     = $entityManager;
        $this->tco01101DetalhesServicoService  = $tco01101DetalhesServicoService;
    }


    /**
     * Salva alteração no registro de serviço
     * da proposta.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function saveEditServicoProposta(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();

        if($this->isValidQuantServProp($row)){
            $this->tco01101DetalhesServicoService->saveEditServicoProposta($request);
        } else {
            $response->setError(new Error('TCO01100_INVALID_QUANT_REF_EST', 1));
        }
    }

    /**
     * Verifica validade das quantidades de refeições
     * estimadas para o serviço correspondente.
     * 
     * @param {DTO\Row} $row Registro a ser verificado.
     * 
     * @return {Boolean}  'true' caso valido e 'false'
     *  caso invalido.
     */
    private function isValidQuantServProp($row){
        $totalDiasTrab = 0;
        $totalRefEstimadas = 0;

        if ($row->get('QTSEGSEX')){
            $totalDiasTrab += 21;
            $totalRefEstimadas += $row->get('QTSEGSEX')*21;
        } 
        if ($row->get('QTSABADO')) {
            $totalDiasTrab += 5;
            $totalRefEstimadas += $row->get('QTSABADO')*5;
        }
        if ($row->get('QTDOMFERI')){
            $totalDiasTrab += 4;
            $totalRefEstimadas += $row->get('QTDOMFERI')*4;
        }
            
        return $totalDiasTrab==$row->get('QTTOTDIAS') &&
                $totalRefEstimadas==$row->get('QTTOTREFEICOES');
    }

    /**
     * Salva alteração nos registros de 
     * tipo de prato do serviço da proposta.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function saveEditTipoPrat(DTO\Request $request, DTO\Response $response){
        $dataset = $request->getDataSet()->getRows();
        $requestParams = array(
            "REQRPATH"   => $request->getRoutePath(),
            "REQMETHOD"  => $request->getMethod(),
            "REQUSERID"  => $request->getUserId()
        );

        $this->tco01101DetalhesServicoService->saveEditTipoPrat($dataset, $requestParams);
    }

    /**
     * Salva alterações de porcionamento e gramatura
     * para o grupo de tipo de prato relacionado.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function saveAddPorcGram(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        
        $requestParams = array(
            "REQRPATH"   => $request->getRoutePath(),
            "REQMETHOD"  => $request->getMethod(),
            "REQUSERID"  => $request->getUserId()
        );

        $this->tco01101DetalhesServicoService->saveAddPorcGram($row, $requestParams);
    }

    /**
     * Executa a rotina de calculo do custo do
     * serviço.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function calcularCustServ(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $procedureResult = $this->tco01101DetalhesServicoService->calcularCustServ($row);

        $response->addDataSet($procedureResult);
    }

    /**
     * Altera estado do serviço de acordo com o
     * passado por parametro.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function setServSituacao(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $nrServicoPropCom = $row->get('NRSERVICOPROPCOM');
        $idSituacao = $row->get('IDSITUACAO');

        $this->tco01101DetalhesServicoService->setSituacaoServPropCom($nrServicoPropCom, $idSituacao);
    }


    /**
     * Salva incidencias alteradas.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function saveIncidencias(DTO\Request $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        
        $requestParams = array(
            "REQRPATH"   => $request->getRoutePath(),
            "REQMETHOD"  => $request->getMethod(),
            "REQUSERID"  => $request->getUserId()
        );

        $this->tco01101DetalhesServicoService->saveIncidencias($rows, $requestParams);
    }

    /**
     * Retorna dataset de marcas disponíveis para troca 
     * do produto.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function getMarcaProduto(DTO\Request $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $result = sizeof($conditions) ? $this->tco01101DetalhesServicoService->getMarcaProduto($conditions) : array();
        $response->addDataSet(new DataSet('tco01101_change_marca_produto', $result));
    }

    /**
     * Altera marca do produto selecionado.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function selectMarca(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $requestParams = array(
            "REQRPATH"   => $request->getRoutePath(),
            "REQMETHOD"  => $request->getMethod(),
            "REQUSERID"  => $request->getUserId()
        );

        $this->tco01101DetalhesServicoService->selectMarca($row, $requestParams);
    }


    /**
     * Limpa marca no produto selecionado.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function clearMarcaProduto(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $this->tco01101DetalhesServicoService->clearMarcaProduto($row);
    }

    /**
     * Rotina de notificar planejamento, salvando
     * dados da tabela "HISTNOTIFPROPCOM".
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function notificarPlan(DTO\Request $request, DTO\Response $response){
        $this->tco01101DetalhesServicoService->notificarPlan($request, $response);
    }

    /**
     * Confirma envio de solicitação 
     * de customização.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function confirmCustomize(DTO\Request $request, DTO\Response $response){
        $this->tco01101DetalhesServicoService->confirmCustomize($request, $response);
    }

    /**
     * Envia resposta ao operador do planejamento.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function enviaResposta(DTO\Request $request, DTO\Response $response){
        $this->tco01101DetalhesServicoService->enviaResposta($request, $response);
    }

    /**
     * Retorna dados para abertura de widget de
     * serviços complementares
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function openServCompl(DTO\Request $request, DTO\Response $response){
        $nrServicoPropCom = $request->getRow()->get('NRSERVICOPROPCOM');
        $servCompl = $this->tco01101DetalhesServicoService->openServCompl($nrServicoPropCom);

        $response->addDataSet(new DataSet('servComplData', $servCompl));
    }

    /**
     * Selecionar serviços complementares
     * para integrar custo do serviço.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function selectServCompl(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $servData = $row->get('servData');
        $servCompl = $row->get('servCompl');

        $selectServCompl = $this->tco01101DetalhesServicoService->selectServCompl($servData, $servCompl);
        
        $response->addDataSet(new DataSet('selectServCompl', array($selectServCompl)));
        
    }
    
    /**
     * Obtém o motivo do último histórico do serviço
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function getUltimaSolicitacao(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        
        $ultima = $this->tco01101DetalhesServicoService->getUltimaSolicitacao($row['NRSERVICOPROPCOM']);
        if (count($ultima) > 0){
            $response->addDataSet(new DataSet('ultima', array($ultima[0]['IDMOTIVO'])));
        }
        
    }
}